# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/controller_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ControllerPreferencesPageWidget(object):
    def setupUi(self, ControllerPreferencesPageWidget):
        ControllerPreferencesPageWidget.setObjectName("ControllerPreferencesPageWidget")
        ControllerPreferencesPageWidget.resize(585, 791)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ControllerPreferencesPageWidget.sizePolicy().hasHeightForWidth())
        ControllerPreferencesPageWidget.setSizePolicy(sizePolicy)
        ControllerPreferencesPageWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ControllerPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiServerPreferenceTabWidget = QtWidgets.QTabWidget(ControllerPreferencesPageWidget)
        self.uiServerPreferenceTabWidget.setObjectName("uiServerPreferenceTabWidget")
        self.uiLocalTabWidget = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalTabWidget.sizePolicy().hasHeightForWidth())
        self.uiLocalTabWidget.setSizePolicy(sizePolicy)
        self.uiLocalTabWidget.setObjectName("uiLocalTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiLocalTabWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiLocalServerAutoStartCheckBox = QtWidgets.QCheckBox(self.uiLocalTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalServerAutoStartCheckBox.sizePolicy().hasHeightForWidth())
        self.uiLocalServerAutoStartCheckBox.setSizePolicy(sizePolicy)
        self.uiLocalServerAutoStartCheckBox.setMinimumSize(QtCore.QSize(0, 0))
        self.uiLocalServerAutoStartCheckBox.setChecked(False)
        self.uiLocalServerAutoStartCheckBox.setObjectName("uiLocalServerAutoStartCheckBox")
        self.verticalLayout.addWidget(self.uiLocalServerAutoStartCheckBox)
        self.uiGeneralSettingsGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiGeneralSettingsGroupBox.setObjectName("uiGeneralSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGeneralSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiLocalServerHostComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerHostComboBox.setObjectName("uiLocalServerHostComboBox")
        self.gridLayout.addWidget(self.uiLocalServerHostComboBox, 5, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiLocalServerPathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPathLineEdit.setObjectName("uiLocalServerPathLineEdit")
        self.horizontalLayout.addWidget(self.uiLocalServerPathLineEdit)
        self.uiLocalServerToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiLocalServerToolButton.setObjectName("uiLocalServerToolButton")
        self.horizontalLayout.addWidget(self.uiLocalServerToolButton)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiUbridgePathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsGroupBox)
        self.uiUbridgePathLineEdit.setObjectName("uiUbridgePathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiUbridgePathLineEdit)
        self.uiUbridgeToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsGroupBox)
        self.uiUbridgeToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiUbridgeToolButton.setObjectName("uiUbridgeToolButton")
        self.horizontalLayout_5.addWidget(self.uiUbridgeToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_5, 3, 0, 1, 1)
        self.uiLocalServerHostLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerHostLabel.setObjectName("uiLocalServerHostLabel")
        self.gridLayout.addWidget(self.uiLocalServerHostLabel, 4, 0, 1, 1)
        self.uiLocalServerPathLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPathLabel.setObjectName("uiLocalServerPathLabel")
        self.gridLayout.addWidget(self.uiLocalServerPathLabel, 0, 0, 1, 1)
        self.uiUbridgePathLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiUbridgePathLabel.setObjectName("uiUbridgePathLabel")
        self.gridLayout.addWidget(self.uiUbridgePathLabel, 2, 0, 1, 1)
        self.uiConsoleConnectionsToAnyIPCheckBox = QtWidgets.QCheckBox(self.uiGeneralSettingsGroupBox)
        self.uiConsoleConnectionsToAnyIPCheckBox.setObjectName("uiConsoleConnectionsToAnyIPCheckBox")
        self.gridLayout.addWidget(self.uiConsoleConnectionsToAnyIPCheckBox, 8, 0, 1, 1)
        self.uiLocalServerPortSpinBox = QtWidgets.QSpinBox(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPortSpinBox.setSuffix(" TCP")
        self.uiLocalServerPortSpinBox.setMaximum(65535)
        self.uiLocalServerPortSpinBox.setProperty("value", 3080)
        self.uiLocalServerPortSpinBox.setObjectName("uiLocalServerPortSpinBox")
        self.gridLayout.addWidget(self.uiLocalServerPortSpinBox, 7, 0, 1, 1)
        self.uiLocalServerPortLabel = QtWidgets.QLabel(self.uiGeneralSettingsGroupBox)
        self.uiLocalServerPortLabel.setObjectName("uiLocalServerPortLabel")
        self.gridLayout.addWidget(self.uiLocalServerPortLabel, 6, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiGeneralSettingsGroupBox)
        self.uiConsolePortRangeGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiConsolePortRangeGroupBox.setObjectName("uiConsolePortRangeGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiConsolePortRangeGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiConsoleStartPortSpinBox = QtWidgets.QSpinBox(self.uiConsolePortRangeGroupBox)
        self.uiConsoleStartPortSpinBox.setSuffix(" TCP")
        self.uiConsoleStartPortSpinBox.setMaximum(65535)
        self.uiConsoleStartPortSpinBox.setProperty("value", 2000)
        self.uiConsoleStartPortSpinBox.setObjectName("uiConsoleStartPortSpinBox")
        self.horizontalLayout_6.addWidget(self.uiConsoleStartPortSpinBox)
        self.uiConsolePortRangeLabel = QtWidgets.QLabel(self.uiConsolePortRangeGroupBox)
        self.uiConsolePortRangeLabel.setObjectName("uiConsolePortRangeLabel")
        self.horizontalLayout_6.addWidget(self.uiConsolePortRangeLabel)
        self.uiConsoleEndPortSpinBox = QtWidgets.QSpinBox(self.uiConsolePortRangeGroupBox)
        self.uiConsoleEndPortSpinBox.setSuffix(" TCP")
        self.uiConsoleEndPortSpinBox.setMaximum(65535)
        self.uiConsoleEndPortSpinBox.setProperty("value", 5000)
        self.uiConsoleEndPortSpinBox.setObjectName("uiConsoleEndPortSpinBox")
        self.horizontalLayout_6.addWidget(self.uiConsoleEndPortSpinBox)
        spacerItem = QtWidgets.QSpacerItem(225, 20, QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiConsolePortRangeGroupBox)
        self.uiUDPPortRangeGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiUDPPortRangeGroupBox.setObjectName("uiUDPPortRangeGroupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiUDPPortRangeGroupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiUDPStartPortSpinBox = QtWidgets.QSpinBox(self.uiUDPPortRangeGroupBox)
        self.uiUDPStartPortSpinBox.setSuffix(" UDP")
        self.uiUDPStartPortSpinBox.setMaximum(65535)
        self.uiUDPStartPortSpinBox.setProperty("value", 10000)
        self.uiUDPStartPortSpinBox.setObjectName("uiUDPStartPortSpinBox")
        self.horizontalLayout_4.addWidget(self.uiUDPStartPortSpinBox)
        self.uiUDPPortRangeLabel = QtWidgets.QLabel(self.uiUDPPortRangeGroupBox)
        self.uiUDPPortRangeLabel.setObjectName("uiUDPPortRangeLabel")
        self.horizontalLayout_4.addWidget(self.uiUDPPortRangeLabel)
        self.uiUDPEndPortSpinBox = QtWidgets.QSpinBox(self.uiUDPPortRangeGroupBox)
        self.uiUDPEndPortSpinBox.setSuffix(" UDP")
        self.uiUDPEndPortSpinBox.setMaximum(65535)
        self.uiUDPEndPortSpinBox.setProperty("value", 20000)
        self.uiUDPEndPortSpinBox.setObjectName("uiUDPEndPortSpinBox")
        self.horizontalLayout_4.addWidget(self.uiUDPEndPortSpinBox)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.gridLayout_3.addLayout(self.horizontalLayout_4, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiUDPPortRangeGroupBox)
        self.uiRemoteMainServerGroupBox = QtWidgets.QGroupBox(self.uiLocalTabWidget)
        self.uiRemoteMainServerGroupBox.setObjectName("uiRemoteMainServerGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiRemoteMainServerGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_2 = QtWidgets.QLabel(self.uiRemoteMainServerGroupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout_4.addWidget(self.label_2, 1, 0, 1, 1)
        self.uiRemoteMainServerUserLineEdit = QtWidgets.QLineEdit(self.uiRemoteMainServerGroupBox)
        self.uiRemoteMainServerUserLineEdit.setObjectName("uiRemoteMainServerUserLineEdit")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerUserLineEdit, 3, 2, 1, 1)
        self.uiRemoteMainServerPasswordLineEdit = QtWidgets.QLineEdit(self.uiRemoteMainServerGroupBox)
        self.uiRemoteMainServerPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiRemoteMainServerPasswordLineEdit.setObjectName("uiRemoteMainServerPasswordLineEdit")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerPasswordLineEdit, 4, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.uiRemoteMainServerGroupBox)
        self.label_5.setObjectName("label_5")
        self.gridLayout_4.addWidget(self.label_5, 4, 0, 1, 2)
        self.uiRemoteMainServerProtocolLabel = QtWidgets.QLabel(self.uiRemoteMainServerGroupBox)
        self.uiRemoteMainServerProtocolLabel.setObjectName("uiRemoteMainServerProtocolLabel")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerProtocolLabel, 0, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.uiRemoteMainServerGroupBox)
        self.label_4.setObjectName("label_4")
        self.gridLayout_4.addWidget(self.label_4, 3, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.uiRemoteMainServerGroupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 2, 0, 1, 1)
        self.uiRemoteMainServerPortSpinBox = QtWidgets.QSpinBox(self.uiRemoteMainServerGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteMainServerPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteMainServerPortSpinBox.setSizePolicy(sizePolicy)
        self.uiRemoteMainServerPortSpinBox.setMaximum(65535)
        self.uiRemoteMainServerPortSpinBox.setProperty("value", 3080)
        self.uiRemoteMainServerPortSpinBox.setObjectName("uiRemoteMainServerPortSpinBox")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerPortSpinBox, 2, 2, 1, 1)
        self.uiRemoteMainServerHostLineEdit = QtWidgets.QLineEdit(self.uiRemoteMainServerGroupBox)
        self.uiRemoteMainServerHostLineEdit.setObjectName("uiRemoteMainServerHostLineEdit")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerHostLineEdit, 1, 2, 1, 1)
        self.uiRemoteMainServerProtocolComboBox = QtWidgets.QComboBox(self.uiRemoteMainServerGroupBox)
        self.uiRemoteMainServerProtocolComboBox.setObjectName("uiRemoteMainServerProtocolComboBox")
        self.uiRemoteMainServerProtocolComboBox.addItem("")
        self.uiRemoteMainServerProtocolComboBox.addItem("")
        self.gridLayout_4.addWidget(self.uiRemoteMainServerProtocolComboBox, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.uiRemoteMainServerGroupBox)
        spacerItem2 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.uiGeneralSettingsGroupBox.raise_()
        self.uiConsolePortRangeGroupBox.raise_()
        self.uiUDPPortRangeGroupBox.raise_()
        self.uiLocalServerAutoStartCheckBox.raise_()
        self.uiRemoteMainServerGroupBox.raise_()
        self.uiServerPreferenceTabWidget.addTab(self.uiLocalTabWidget, "")
        self.uiRemoteTabWidget = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteTabWidget.sizePolicy().hasHeightForWidth())
        self.uiRemoteTabWidget.setSizePolicy(sizePolicy)
        self.uiRemoteTabWidget.setObjectName("uiRemoteTabWidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiRemoteTabWidget)
        self.gridLayout_5.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiRemoteServersTreeWidget = QtWidgets.QTreeWidget(self.uiRemoteTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersTreeWidget.setSizePolicy(sizePolicy)
        self.uiRemoteServersTreeWidget.setColumnCount(5)
        self.uiRemoteServersTreeWidget.setObjectName("uiRemoteServersTreeWidget")
        self.uiRemoteServersTreeWidget.headerItem().setText(1, "Protocol")
        self.uiRemoteServersTreeWidget.headerItem().setText(2, "Host")
        self.uiRemoteServersTreeWidget.headerItem().setText(3, "Port")
        self.gridLayout_5.addWidget(self.uiRemoteServersTreeWidget, 0, 0, 1, 2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiAddRemoteServerPushButton = QtWidgets.QPushButton(self.uiRemoteTabWidget)
        self.uiAddRemoteServerPushButton.setObjectName("uiAddRemoteServerPushButton")
        self.horizontalLayout_3.addWidget(self.uiAddRemoteServerPushButton)
        self.uiUpdateRemoteServerPushButton = QtWidgets.QPushButton(self.uiRemoteTabWidget)
        self.uiUpdateRemoteServerPushButton.setEnabled(False)
        self.uiUpdateRemoteServerPushButton.setObjectName("uiUpdateRemoteServerPushButton")
        self.horizontalLayout_3.addWidget(self.uiUpdateRemoteServerPushButton)
        self.uiDeleteRemoteServerPushButton = QtWidgets.QPushButton(self.uiRemoteTabWidget)
        self.uiDeleteRemoteServerPushButton.setEnabled(False)
        self.uiDeleteRemoteServerPushButton.setObjectName("uiDeleteRemoteServerPushButton")
        self.horizontalLayout_3.addWidget(self.uiDeleteRemoteServerPushButton)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.gridLayout_5.addLayout(self.horizontalLayout_3, 1, 0, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem4, 3, 0, 1, 1)
        self.uiDynamicComputeAllocationCheckBox = QtWidgets.QCheckBox(self.uiRemoteTabWidget)
        self.uiDynamicComputeAllocationCheckBox.setObjectName("uiDynamicComputeAllocationCheckBox")
        self.gridLayout_5.addWidget(self.uiDynamicComputeAllocationCheckBox, 2, 0, 1, 1)
        self.uiRemoteServersTreeWidget.raise_()
        self.uiDynamicComputeAllocationCheckBox.raise_()
        self.uiServerPreferenceTabWidget.addTab(self.uiRemoteTabWidget, "")
        self.verticalLayout_2.addWidget(self.uiServerPreferenceTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(ControllerPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(ControllerPreferencesPageWidget)
        self.uiServerPreferenceTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ControllerPreferencesPageWidget)
        ControllerPreferencesPageWidget.setTabOrder(self.uiServerPreferenceTabWidget, self.uiLocalServerAutoStartCheckBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiLocalServerAutoStartCheckBox, self.uiLocalServerPathLineEdit)
        ControllerPreferencesPageWidget.setTabOrder(self.uiLocalServerPathLineEdit, self.uiLocalServerToolButton)
        ControllerPreferencesPageWidget.setTabOrder(self.uiLocalServerToolButton, self.uiUbridgePathLineEdit)
        ControllerPreferencesPageWidget.setTabOrder(self.uiUbridgePathLineEdit, self.uiUbridgeToolButton)
        ControllerPreferencesPageWidget.setTabOrder(self.uiUbridgeToolButton, self.uiLocalServerHostComboBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiLocalServerHostComboBox, self.uiLocalServerPortSpinBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiLocalServerPortSpinBox, self.uiConsoleConnectionsToAnyIPCheckBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiConsoleConnectionsToAnyIPCheckBox, self.uiConsoleStartPortSpinBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiConsoleStartPortSpinBox, self.uiConsoleEndPortSpinBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiConsoleEndPortSpinBox, self.uiUDPStartPortSpinBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiUDPStartPortSpinBox, self.uiUDPEndPortSpinBox)
        ControllerPreferencesPageWidget.setTabOrder(self.uiUDPEndPortSpinBox, self.uiRemoteServersTreeWidget)
        ControllerPreferencesPageWidget.setTabOrder(self.uiRemoteServersTreeWidget, self.uiAddRemoteServerPushButton)
        ControllerPreferencesPageWidget.setTabOrder(self.uiAddRemoteServerPushButton, self.uiDeleteRemoteServerPushButton)

    def retranslateUi(self, ControllerPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        ControllerPreferencesPageWidget.setWindowTitle(_translate("ControllerPreferencesPageWidget", "Controller"))
        self.uiLocalServerAutoStartCheckBox.setText(_translate("ControllerPreferencesPageWidget", "Enable local controller"))
        self.uiGeneralSettingsGroupBox.setTitle(_translate("ControllerPreferencesPageWidget", "General settings"))
        self.uiLocalServerToolButton.setText(_translate("ControllerPreferencesPageWidget", "&Browse..."))
        self.uiUbridgeToolButton.setText(_translate("ControllerPreferencesPageWidget", "&Browse..."))
        self.uiLocalServerHostLabel.setText(_translate("ControllerPreferencesPageWidget", "Host binding:"))
        self.uiLocalServerPathLabel.setText(_translate("ControllerPreferencesPageWidget", "Server path:"))
        self.uiUbridgePathLabel.setText(_translate("ControllerPreferencesPageWidget", "Ubridge path:"))
        self.uiConsoleConnectionsToAnyIPCheckBox.setText(_translate("ControllerPreferencesPageWidget", "Allow console connections to any local IP address"))
        self.uiLocalServerPortLabel.setText(_translate("ControllerPreferencesPageWidget", "Port:"))
        self.uiConsolePortRangeGroupBox.setTitle(_translate("ControllerPreferencesPageWidget", "Console port range (5900 => 6000 is shared with VNC)"))
        self.uiConsolePortRangeLabel.setText(_translate("ControllerPreferencesPageWidget", "to"))
        self.uiUDPPortRangeGroupBox.setTitle(_translate("ControllerPreferencesPageWidget", "UDP tunneling port range"))
        self.uiUDPPortRangeLabel.setText(_translate("ControllerPreferencesPageWidget", "to"))
        self.uiRemoteMainServerGroupBox.setTitle(_translate("ControllerPreferencesPageWidget", "Remote controller"))
        self.label_2.setText(_translate("ControllerPreferencesPageWidget", "Host:"))
        self.label_5.setText(_translate("ControllerPreferencesPageWidget", "Password:"))
        self.uiRemoteMainServerProtocolLabel.setText(_translate("ControllerPreferencesPageWidget", "Protocol:"))
        self.label_4.setText(_translate("ControllerPreferencesPageWidget", "Username:"))
        self.label_3.setText(_translate("ControllerPreferencesPageWidget", "Port:"))
        self.uiRemoteMainServerPortSpinBox.setSuffix(_translate("ControllerPreferencesPageWidget", " TCP"))
        self.uiRemoteMainServerProtocolComboBox.setItemText(0, _translate("ControllerPreferencesPageWidget", "HTTP"))
        self.uiRemoteMainServerProtocolComboBox.setItemText(1, _translate("ControllerPreferencesPageWidget", "HTTPS"))
        self.uiServerPreferenceTabWidget.setTabText(self.uiServerPreferenceTabWidget.indexOf(self.uiLocalTabWidget), _translate("ControllerPreferencesPageWidget", "Controller"))
        self.uiRemoteServersTreeWidget.headerItem().setText(0, _translate("ControllerPreferencesPageWidget", "Name"))
        self.uiRemoteServersTreeWidget.headerItem().setText(4, _translate("ControllerPreferencesPageWidget", "Username"))
        self.uiAddRemoteServerPushButton.setText(_translate("ControllerPreferencesPageWidget", "&Add"))
        self.uiUpdateRemoteServerPushButton.setText(_translate("ControllerPreferencesPageWidget", "Edit"))
        self.uiDeleteRemoteServerPushButton.setText(_translate("ControllerPreferencesPageWidget", "&Delete"))
        self.uiDynamicComputeAllocationCheckBox.setText(_translate("ControllerPreferencesPageWidget", "Allow controller to dynamically allocate computes"))
        self.uiServerPreferenceTabWidget.setTabText(self.uiServerPreferenceTabWidget.indexOf(self.uiRemoteTabWidget), _translate("ControllerPreferencesPageWidget", "Remote computes"))
        self.uiRestoreDefaultsPushButton.setText(_translate("ControllerPreferencesPageWidget", "Restore defaults"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/setup_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SetupWizard(object):
    def setupUi(self, SetupWizard):
        SetupWizard.setObjectName("SetupWizard")
        SetupWizard.resize(603, 287)
        SetupWizard.setModal(True)
        SetupWizard.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        SetupWizard.setOptions(QtWidgets.QWizard.NoBackButtonOnStartPage)
        self.uiControllerWizardPage = QtWidgets.QWizardPage()
        self.uiControllerWizardPage.setObjectName("uiControllerWizardPage")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiControllerWizardPage)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiLocalControllerRadioButton = QtWidgets.QRadioButton(self.uiControllerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalControllerRadioButton.sizePolicy().hasHeightForWidth())
        self.uiLocalControllerRadioButton.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.uiLocalControllerRadioButton.setFont(font)
        self.uiLocalControllerRadioButton.setChecked(True)
        self.uiLocalControllerRadioButton.setObjectName("uiLocalControllerRadioButton")
        self.verticalLayout_4.addWidget(self.uiLocalControllerRadioButton)
        self.uiRemoteControllerRadioButton = QtWidgets.QRadioButton(self.uiControllerWizardPage)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.uiRemoteControllerRadioButton.setFont(font)
        self.uiRemoteControllerRadioButton.setObjectName("uiRemoteControllerRadioButton")
        self.verticalLayout_4.addWidget(self.uiRemoteControllerRadioButton)
        spacerItem = QtWidgets.QSpacerItem(20, 212, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        SetupWizard.addPage(self.uiControllerWizardPage)
        self.uiLocalControllerWizardPage = QtWidgets.QWizardPage()
        self.uiLocalControllerWizardPage.setObjectName("uiLocalControllerWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiLocalControllerWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiLocalServerPathLabel = QtWidgets.QLabel(self.uiLocalControllerWizardPage)
        self.uiLocalServerPathLabel.setObjectName("uiLocalServerPathLabel")
        self.gridLayout_2.addWidget(self.uiLocalServerPathLabel, 0, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiLocalServerPathLineEdit = QtWidgets.QLineEdit(self.uiLocalControllerWizardPage)
        self.uiLocalServerPathLineEdit.setObjectName("uiLocalServerPathLineEdit")
        self.horizontalLayout_3.addWidget(self.uiLocalServerPathLineEdit)
        self.uiLocalServerToolButton = QtWidgets.QToolButton(self.uiLocalControllerWizardPage)
        self.uiLocalServerToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiLocalServerToolButton.setObjectName("uiLocalServerToolButton")
        self.horizontalLayout_3.addWidget(self.uiLocalServerToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 0, 1, 1, 1)
        self.uiLocalServerHostLabel = QtWidgets.QLabel(self.uiLocalControllerWizardPage)
        self.uiLocalServerHostLabel.setObjectName("uiLocalServerHostLabel")
        self.gridLayout_2.addWidget(self.uiLocalServerHostLabel, 1, 0, 1, 1)
        self.uiLocalServerHostComboBox = QtWidgets.QComboBox(self.uiLocalControllerWizardPage)
        self.uiLocalServerHostComboBox.setObjectName("uiLocalServerHostComboBox")
        self.gridLayout_2.addWidget(self.uiLocalServerHostComboBox, 1, 1, 1, 1)
        self.uiLocalServerPortLabel = QtWidgets.QLabel(self.uiLocalControllerWizardPage)
        self.uiLocalServerPortLabel.setObjectName("uiLocalServerPortLabel")
        self.gridLayout_2.addWidget(self.uiLocalServerPortLabel, 2, 0, 1, 1)
        self.uiLocalServerPortSpinBox = QtWidgets.QSpinBox(self.uiLocalControllerWizardPage)
        self.uiLocalServerPortSpinBox.setSuffix(" TCP")
        self.uiLocalServerPortSpinBox.setMaximum(65535)
        self.uiLocalServerPortSpinBox.setProperty("value", 3080)
        self.uiLocalServerPortSpinBox.setObjectName("uiLocalServerPortSpinBox")
        self.gridLayout_2.addWidget(self.uiLocalServerPortSpinBox, 2, 1, 1, 1)
        SetupWizard.addPage(self.uiLocalControllerWizardPage)
        self.uiRemoteControllerWizardPage = QtWidgets.QWizardPage()
        self.uiRemoteControllerWizardPage.setObjectName("uiRemoteControllerWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiRemoteControllerWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.label_3 = QtWidgets.QLabel(self.uiRemoteControllerWizardPage)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.uiRemoteMainServerPasswordLineEdit = QtWidgets.QLineEdit(self.uiRemoteControllerWizardPage)
        self.uiRemoteMainServerPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiRemoteMainServerPasswordLineEdit.setObjectName("uiRemoteMainServerPasswordLineEdit")
        self.gridLayout.addWidget(self.uiRemoteMainServerPasswordLineEdit, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.uiRemoteControllerWizardPage)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 3, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.uiRemoteControllerWizardPage)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)
        self.uiRemoteMainServerUserLineEdit = QtWidgets.QLineEdit(self.uiRemoteControllerWizardPage)
        self.uiRemoteMainServerUserLineEdit.setObjectName("uiRemoteMainServerUserLineEdit")
        self.gridLayout.addWidget(self.uiRemoteMainServerUserLineEdit, 2, 1, 1, 1)
        self.uiRemoteMainServerPortSpinBox = QtWidgets.QSpinBox(self.uiRemoteControllerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteMainServerPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteMainServerPortSpinBox.setSizePolicy(sizePolicy)
        self.uiRemoteMainServerPortSpinBox.setMaximum(65535)
        self.uiRemoteMainServerPortSpinBox.setProperty("value", 3080)
        self.uiRemoteMainServerPortSpinBox.setObjectName("uiRemoteMainServerPortSpinBox")
        self.gridLayout.addWidget(self.uiRemoteMainServerPortSpinBox, 1, 1, 1, 1)
        self.uiRemoteMainServerHostLineEdit = QtWidgets.QLineEdit(self.uiRemoteControllerWizardPage)
        self.uiRemoteMainServerHostLineEdit.setObjectName("uiRemoteMainServerHostLineEdit")
        self.gridLayout.addWidget(self.uiRemoteMainServerHostLineEdit, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.uiRemoteControllerWizardPage)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 2, 0, 1, 1)
        SetupWizard.addPage(self.uiRemoteControllerWizardPage)
        self.uiSummaryWizardPage = QtWidgets.QWizardPage()
        self.uiSummaryWizardPage.setObjectName("uiSummaryWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiSummaryWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiSummaryTreeWidget = QtWidgets.QTreeWidget(self.uiSummaryWizardPage)
        self.uiSummaryTreeWidget.setRootIsDecorated(False)
        self.uiSummaryTreeWidget.setObjectName("uiSummaryTreeWidget")
        self.uiSummaryTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiSummaryTreeWidget)
        self.uiShowCheckBox = QtWidgets.QCheckBox(self.uiSummaryWizardPage)
        self.uiShowCheckBox.setChecked(True)
        self.uiShowCheckBox.setObjectName("uiShowCheckBox")
        self.verticalLayout.addWidget(self.uiShowCheckBox)
        SetupWizard.addPage(self.uiSummaryWizardPage)

        self.retranslateUi(SetupWizard)
        QtCore.QMetaObject.connectSlotsByName(SetupWizard)

    def retranslateUi(self, SetupWizard):
        _translate = QtCore.QCoreApplication.translate
        SetupWizard.setWindowTitle(_translate("SetupWizard", "Setup Wizard"))
        self.uiControllerWizardPage.setTitle(_translate("SetupWizard", "Controller"))
        self.uiControllerWizardPage.setSubTitle(_translate("SetupWizard", "Please select an option to connect to the GNS3 controller"))
        self.uiLocalControllerRadioButton.setToolTip(_translate("SetupWizard", "A local controller runs on the same machine as this graphical interface"))
        self.uiLocalControllerRadioButton.setText(_translate("SetupWizard", "Start and connect to a local controller (Linux only)"))
        self.uiRemoteControllerRadioButton.setToolTip(_translate("SetupWizard", "A remote controller can run in a Virtual Machine (VM) or on a remote server"))
        self.uiRemoteControllerRadioButton.setText(_translate("SetupWizard", "Connect to a remote controller"))
        self.uiLocalControllerWizardPage.setTitle(_translate("SetupWizard", "Local controller configuration"))
        self.uiLocalControllerWizardPage.setSubTitle(_translate("SetupWizard", "Please configure the following local server settings"))
        self.uiLocalServerPathLabel.setText(_translate("SetupWizard", "Server path:"))
        self.uiLocalServerToolButton.setText(_translate("SetupWizard", "&Browse..."))
        self.uiLocalServerHostLabel.setText(_translate("SetupWizard", "Host binding:"))
        self.uiLocalServerPortLabel.setText(_translate("SetupWizard", "Port:"))
        self.uiRemoteControllerWizardPage.setTitle(_translate("SetupWizard", "Remote controller"))
        self.uiRemoteControllerWizardPage.setSubTitle(_translate("SetupWizard", "Please configure the settings to connect to a remote GNS3 controller"))
        self.label_3.setText(_translate("SetupWizard", "Host:"))
        self.label_6.setText(_translate("SetupWizard", "Password:"))
        self.label_4.setText(_translate("SetupWizard", "Port:"))
        self.uiRemoteMainServerPortSpinBox.setSuffix(_translate("SetupWizard", " TCP"))
        self.label_5.setText(_translate("SetupWizard", "Username:"))
        self.uiSummaryWizardPage.setTitle(_translate("SetupWizard", "Summary"))
        self.uiSummaryWizardPage.setSubTitle(_translate("SetupWizard", "The controller has been configured, please see the summary of the settings below"))
        self.uiSummaryTreeWidget.headerItem().setText(0, _translate("SetupWizard", "1"))
        self.uiSummaryTreeWidget.headerItem().setText(1, _translate("SetupWizard", "2"))
        self.uiShowCheckBox.setText(_translate("SetupWizard", "Don\'t show this setup wizard again"))
from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Docker images.
"""

from gns3.qt import QtWidgets
from gns3.node import Node
from gns3.dialogs.custom_adapters_configuration_dialog import CustomAdaptersConfigurationDialog

from ..ui.docker_vm_configuration_page_ui import Ui_dockerVMConfigPageWidget
from ....dialogs.file_editor_dialog import FileEditorDialog
from ....dialogs.node_properties_dialog import ConfigurationError
from ....dialogs.symbol_selection_dialog import SymbolSelectionDialog


class DockerVMConfigurationPage(QtWidgets.QWidget, Ui_dockerVMConfigPageWidget):
    """
    QWidget configuration page for Docker images
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._settings = None
        self._custom_adapters = []

        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)
        self.uiNetworkConfigEditButton.released.connect(self._networkConfigEditSlot)
        self.uiCustomAdaptersConfigurationPushButton.clicked.connect(self._customAdaptersConfigurationSlot)

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def _customAdaptersConfigurationSlot(self):
        """
        Slot to open the custom adapters configuration dialog
        """

        if self._node:
            adapters = self._settings["adapters"]
        else:
            adapters = self.uiAdapterSpinBox.value()

        ports = []
        for adapter_number in range(0, adapters):
            port_name = "eth{}".format(adapter_number)
            ports.append(port_name)

        dialog = CustomAdaptersConfigurationDialog(ports, self._custom_adapters, parent=self)
        dialog.show()
        dialog.exec_()

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the Docker container settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of images
        """

        if node:
            self._node = node
            self._settings = settings
        else:
            self._node = None

        self.uiCMDLineEdit.setText(settings["start_command"])
        self.uiEnvironmentTextEdit.setText(settings["environment"])
        self.uiConsoleTypeComboBox.setCurrentIndex(self.uiConsoleTypeComboBox.findText(settings["console_type"]))
        self.uiConsoleAutoStartCheckBox.setChecked(settings["console_auto_start"])
        self.uiAuxTypeComboBox.setCurrentIndex(self.uiAuxTypeComboBox.findText(settings["aux_type"]))
        self.uiConsoleResolutionComboBox.setCurrentIndex(self.uiConsoleResolutionComboBox.findText(settings["console_resolution"]))
        self.uiConsoleHttpPortSpinBox.setValue(settings["console_http_port"])
        self.uiHttpConsolePathLineEdit.setText(settings["console_http_path"])
        self.uiExtraHostsTextEdit.setPlainText(settings["extra_hosts"])
        self.uiExtraVolumeTextEdit.setPlainText("\n".join(settings["extra_volumes"]))
        self.uiMaxMemorySpinBox.setValue(settings["memory"])
        self.uiMaxCPUsDoubleSpinBox.setValue(settings["cpus"])

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
            self.uiAdapterSpinBox.setValue(settings["adapters"])
            self._custom_adapters = settings["custom_adapters"].copy()
        else:
            self.uiNameLabel.hide()
            self.uiNameLineEdit.hide()
            self.uiCMDLabel.hide()
            self.uiCMDLineEdit.hide()
            self.uiAdapterLabel.hide()
            self.uiAdapterSpinBox.hide()
            self.uiCategoryComboBox.hide()
            self.uiCustomAdaptersLabel.hide()
            self.uiCustomAdaptersConfigurationPushButton.hide()

        if not node:
            # these are template settings

            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)
            self.uiNetworkConfigEditButton.hide()
            self.uiNetworkConfigLabel.hide()
        else:
            self._node = node
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()

            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()

            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()

        self.uiUsageTextEdit.setPlainText(settings["usage"])

    def _networkConfigEditSlot(self):

        dialog = FileEditorDialog(self._node, self._node.configFiles()[0])
        dialog.setModal(True)
        self.stackUnder(dialog)
        dialog.show()

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the Docker container settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        settings["start_command"] = self.uiCMDLineEdit.text()
        settings["environment"] = self.uiEnvironmentTextEdit.toPlainText()
        settings["console_type"] = self.uiConsoleTypeComboBox.currentText()
        settings["console_auto_start"] = self.uiConsoleAutoStartCheckBox.isChecked()
        settings["aux_type"] = self.uiAuxTypeComboBox.currentText()
        settings["console_resolution"] = self.uiConsoleResolutionComboBox.currentText()
        settings["console_http_port"] = self.uiConsoleHttpPortSpinBox.value()
        settings["console_http_path"] = self.uiHttpConsolePathLineEdit.text()
        settings["extra_hosts"] = self.uiExtraHostsTextEdit.toPlainText()
        # only tidy input here, validation is performed server side
        settings["extra_volumes"] = [ y for x in self.uiExtraVolumeTextEdit.toPlainText().split("\n") for y in [ x.strip() ] if y ]
        settings["memory"] = self.uiMaxMemorySpinBox.value()
        settings["cpus"] = self.uiMaxCPUsDoubleSpinBox.value()

        if not group:
            adapters = self.uiAdapterSpinBox.value()
            if node:
                if settings["adapters"] != adapters:
                    # check if the adapters settings have changed
                    node_ports = node.ports()
                    for node_port in node_ports:
                        if not node_port.isFree():
                            QtWidgets.QMessageBox.critical(self, node.name(), "Changing the number of adapters while links are connected isn't supported yet! Please delete all the links first.")
                            raise ConfigurationError()

            settings["adapters"] = adapters
            settings["custom_adapters"] = self._custom_adapters.copy()

            name = self.uiNameLineEdit.text()
            if not name:
                QtWidgets.QMessageBox.critical(self, "Name", "Docker name cannot be empty!")
            else:
                settings["name"] = name

        if not node:
            # these are template settings
            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())

            # save the default name format
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            settings["symbol"] = symbol_path

        settings["usage"] = self.uiUsageTextEdit.toPlainText()
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VirtualBox module implementation.
"""

import os
import sys
import shutil

from gns3.local_config import LocalConfig
from gns3.controller import Controller
from gns3.template_manager import TemplateManager
from gns3.template import Template

from ..module import Module
from .virtualbox_vm import VirtualBoxVM
from .settings import VBOX_SETTINGS, VBOX_VM_SETTINGS

import logging
log = logging.getLogger(__name__)


class VirtualBox(Module):
    """
    VirtualBox module.
    """

    def __init__(self):
        super().__init__()
        self._loadSettings()

    @staticmethod
    def _findVBoxManage(self):
        """
        Finds the VBoxManage path.

        :return: path to VBoxManage
        """

        vboxmanage_path = None
        if sys.platform.startswith("win"):
            if "VBOX_INSTALL_PATH" in os.environ:
                vboxmanage_path_windows = os.path.join(os.environ["VBOX_INSTALL_PATH"], "VBoxManage.exe")
                if os.path.exists(vboxmanage_path_windows):
                    vboxmanage_path = vboxmanage_path_windows
            elif "VBOX_MSI_INSTALL_PATH" in os.environ:
                vboxmanage_path_windows = os.path.join(os.environ["VBOX_MSI_INSTALL_PATH"], "VBoxManage.exe")
                if os.path.exists(vboxmanage_path_windows):
                    vboxmanage_path = vboxmanage_path_windows
        elif sys.platform.startswith("darwin"):
            vboxmanage_path_osx = "/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"
            if os.path.exists(vboxmanage_path_osx):
                vboxmanage_path = vboxmanage_path_osx

        if vboxmanage_path is None:
            vboxmanage_path = shutil.which("vboxmanage")

        if vboxmanage_path is None:
            return ""
        return os.path.abspath(vboxmanage_path)

    def _loadSettings(self):
        """
        Loads the settings from the server settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, VBOX_SETTINGS)

        if not os.path.exists(self._settings["vboxmanage_path"]):
            self._settings["vboxmanage_path"] = self._findVBoxManage(self)

        # migrate VM settings to the controller (templates are managed on server side starting with version 2.0)
        Controller.instance().connected_signal.connect(self._migrateOldVMs)

    def _migrateOldVMs(self):
        """
        Migrate local VM settings to the controller.
        """

        if self._settings.get("vms"):
            templates = []
            for vm in self._settings.get("vms"):
                vm_settings = VBOX_VM_SETTINGS.copy()
                vm_settings.update(vm)
                templates.append(Template(vm_settings))
            TemplateManager.instance().updateList(templates)
            self._settings["vms"] = []
            self._saveSettings()

    def _saveSettings(self):
        """
        Saves the settings to the server settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # FIXME: handle server side config
        # save some settings to the local server config file
        # if self._settings["vboxmanage_path"]:
        #     server_settings = {"vboxmanage_path": os.path.normpath(self._settings["vboxmanage_path"])}
        #     LocalServerConfig.instance().saveSettings(self.__class__.__name__, server_settings)

    @staticmethod
    def configurationPage():
        """
        Returns the configuration page for this module.

        :returns: QWidget object
        """

        from .pages.virtualbox_vm_configuration_page import VirtualBoxVMConfigurationPage
        return VirtualBoxVMConfigurationPage

    @staticmethod
    def getNodeClass(node_type, platform=None):
        """
        Returns the class corresponding to node type.

        :param node_type: name of the node
        :param platform: not used

        :returns: class or None
        """

        if node_type == "virtualbox":
            return VirtualBoxVM
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [VirtualBoxVM]

    @staticmethod
    def preferencePages():
        """
        Returns the preference pages for this module.

        :returns: QWidget object list
        """

        from .pages.virtualbox_preferences_page import VirtualBoxPreferencesPage
        from .pages.virtualbox_vm_preferences_page import VirtualBoxVMPreferencesPage
        return [VirtualBoxPreferencesPage, VirtualBoxVMPreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of VirtualBox module.

        :returns: instance of VirtualBox
        """

        if not hasattr(VirtualBox, "_instance"):
            VirtualBox._instance = VirtualBox()
        return VirtualBox._instance

    def __str__(self):
        """
        Returns the module name.
        """

        return "virtualbox"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/project_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.13.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ProjectDialog(object):
    def setupUi(self, ProjectDialog):
        ProjectDialog.setObjectName("ProjectDialog")
        ProjectDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        ProjectDialog.resize(823, 367)
        ProjectDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(ProjectDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiProjectTabWidget = QtWidgets.QTabWidget(ProjectDialog)
        self.uiProjectTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.uiProjectTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.uiProjectTabWidget.setObjectName("uiProjectTabWidget")
        self.uiNewProjectTab = QtWidgets.QWidget()
        self.uiNewProjectTab.setObjectName("uiNewProjectTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiNewProjectTab)
        self.verticalLayout_3.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.uiNewProjectTab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiLocationLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiLocationLineEdit.setObjectName("uiLocationLineEdit")
        self.horizontalLayout_3.addWidget(self.uiLocationLineEdit)
        self.uiLocationBrowserToolButton = QtWidgets.QToolButton(self.groupBox)
        self.uiLocationBrowserToolButton.setObjectName("uiLocationBrowserToolButton")
        self.horizontalLayout_3.addWidget(self.uiLocationBrowserToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_3, 1, 1, 1, 1)
        self.uiNameLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNameLabel.sizePolicy().hasHeightForWidth())
        self.uiNameLabel.setSizePolicy(sizePolicy)
        self.uiNameLabel.setTextFormat(QtCore.Qt.AutoText)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiLocationLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocationLabel.sizePolicy().hasHeightForWidth())
        self.uiLocationLabel.setSizePolicy(sizePolicy)
        self.uiLocationLabel.setObjectName("uiLocationLabel")
        self.gridLayout.addWidget(self.uiLocationLabel, 1, 0, 1, 1)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.uiOpenProjectGroupBox = QtWidgets.QGroupBox(self.uiNewProjectTab)
        self.uiOpenProjectGroupBox.setObjectName("uiOpenProjectGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiOpenProjectGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiOpenProjectPushButton = QtWidgets.QPushButton(self.uiOpenProjectGroupBox)
        self.uiOpenProjectPushButton.setObjectName("uiOpenProjectPushButton")
        self.horizontalLayout_2.addWidget(self.uiOpenProjectPushButton)
        self.uiRecentProjectsPushButton = QtWidgets.QPushButton(self.uiOpenProjectGroupBox)
        self.uiRecentProjectsPushButton.setObjectName("uiRecentProjectsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRecentProjectsPushButton)
        spacerItem = QtWidgets.QSpacerItem(239, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 1, 0, 1, 1)
        self.verticalLayout_3.addWidget(self.uiOpenProjectGroupBox)
        self.uiProjectTabWidget.addTab(self.uiNewProjectTab, "")
        self.uiProjectsLibraryTab = QtWidgets.QWidget()
        self.uiProjectsLibraryTab.setObjectName("uiProjectsLibraryTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiProjectsLibraryTab)
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiProjectsTreeWidget = QtWidgets.QTreeWidget(self.uiProjectsLibraryTab)
        self.uiProjectsTreeWidget.setAlternatingRowColors(True)
        self.uiProjectsTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiProjectsTreeWidget.setObjectName("uiProjectsTreeWidget")
        self.uiProjectsTreeWidget.header().setSortIndicatorShown(True)
        self.verticalLayout_2.addWidget(self.uiProjectsTreeWidget)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiDeleteProjectButton = QtWidgets.QPushButton(self.uiProjectsLibraryTab)
        self.uiDeleteProjectButton.setObjectName("uiDeleteProjectButton")
        self.horizontalLayout_4.addWidget(self.uiDeleteProjectButton)
        self.uiDuplicateProjectPushButton = QtWidgets.QPushButton(self.uiProjectsLibraryTab)
        self.uiDuplicateProjectPushButton.setObjectName("uiDuplicateProjectPushButton")
        self.horizontalLayout_4.addWidget(self.uiDuplicateProjectPushButton)
        self.uiResetMacAddressesCheckBox = QtWidgets.QCheckBox(self.uiProjectsLibraryTab)
        self.uiResetMacAddressesCheckBox.setChecked(True)
        self.uiResetMacAddressesCheckBox.setObjectName("uiResetMacAddressesCheckBox")
        self.horizontalLayout_4.addWidget(self.uiResetMacAddressesCheckBox)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.uiRefreshProjectsPushButton = QtWidgets.QPushButton(self.uiProjectsLibraryTab)
        self.uiRefreshProjectsPushButton.setObjectName("uiRefreshProjectsPushButton")
        self.horizontalLayout_4.addWidget(self.uiRefreshProjectsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.uiProjectTabWidget.addTab(self.uiProjectsLibraryTab, "")
        self.verticalLayout.addWidget(self.uiProjectTabWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiSettingsPushButton = QtWidgets.QPushButton(ProjectDialog)
        self.uiSettingsPushButton.setObjectName("uiSettingsPushButton")
        self.horizontalLayout.addWidget(self.uiSettingsPushButton)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(ProjectDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.horizontalLayout.addWidget(self.uiButtonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(ProjectDialog)
        self.uiProjectTabWidget.setCurrentIndex(0)
        self.uiButtonBox.accepted.connect(ProjectDialog.accept)
        self.uiButtonBox.rejected.connect(ProjectDialog.reject)
        self.uiNameLineEdit.returnPressed.connect(ProjectDialog.accept)
        self.uiLocationLineEdit.returnPressed.connect(ProjectDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(ProjectDialog)
        ProjectDialog.setTabOrder(self.uiProjectTabWidget, self.uiNameLineEdit)
        ProjectDialog.setTabOrder(self.uiNameLineEdit, self.uiLocationLineEdit)
        ProjectDialog.setTabOrder(self.uiLocationLineEdit, self.uiLocationBrowserToolButton)
        ProjectDialog.setTabOrder(self.uiLocationBrowserToolButton, self.uiOpenProjectPushButton)
        ProjectDialog.setTabOrder(self.uiOpenProjectPushButton, self.uiRecentProjectsPushButton)
        ProjectDialog.setTabOrder(self.uiRecentProjectsPushButton, self.uiSettingsPushButton)
        ProjectDialog.setTabOrder(self.uiSettingsPushButton, self.uiProjectsTreeWidget)
        ProjectDialog.setTabOrder(self.uiProjectsTreeWidget, self.uiDeleteProjectButton)
        ProjectDialog.setTabOrder(self.uiDeleteProjectButton, self.uiDuplicateProjectPushButton)
        ProjectDialog.setTabOrder(self.uiDuplicateProjectPushButton, self.uiRefreshProjectsPushButton)

    def retranslateUi(self, ProjectDialog):
        _translate = QtCore.QCoreApplication.translate
        ProjectDialog.setWindowTitle(_translate("ProjectDialog", "Project"))
        self.groupBox.setTitle(_translate("ProjectDialog", "New project"))
        self.uiLocationBrowserToolButton.setText(_translate("ProjectDialog", "Browse..."))
        self.uiNameLabel.setText(_translate("ProjectDialog", "Name:"))
        self.uiLocationLabel.setText(_translate("ProjectDialog", "Location:"))
        self.uiOpenProjectGroupBox.setTitle(_translate("ProjectDialog", "Open project"))
        self.uiOpenProjectPushButton.setText(_translate("ProjectDialog", "&Open a project from disk"))
        self.uiRecentProjectsPushButton.setText(_translate("ProjectDialog", "&Recent projects..."))
        self.uiProjectTabWidget.setTabText(self.uiProjectTabWidget.indexOf(self.uiNewProjectTab), _translate("ProjectDialog", "New project"))
        self.uiProjectsTreeWidget.setSortingEnabled(True)
        self.uiProjectsTreeWidget.headerItem().setText(0, _translate("ProjectDialog", "Name"))
        self.uiProjectsTreeWidget.headerItem().setText(1, _translate("ProjectDialog", "Status"))
        self.uiProjectsTreeWidget.headerItem().setText(2, _translate("ProjectDialog", "Path"))
        self.uiDeleteProjectButton.setText(_translate("ProjectDialog", "Delete"))
        self.uiDuplicateProjectPushButton.setText(_translate("ProjectDialog", "Duplicate"))
        self.uiResetMacAddressesCheckBox.setText(_translate("ProjectDialog", "&Reset MAC addresses"))
        self.uiRefreshProjectsPushButton.setText(_translate("ProjectDialog", "Refresh list"))
        self.uiProjectTabWidget.setTabText(self.uiProjectTabWidget.indexOf(self.uiProjectsLibraryTab), _translate("ProjectDialog", "Projects library"))
        self.uiSettingsPushButton.setText(_translate("ProjectDialog", "Settings"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/ethernet_switch_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_EthernetSwitchPreferencesPageWidget(object):
    def setupUi(self, EthernetSwitchPreferencesPageWidget):
        EthernetSwitchPreferencesPageWidget.setObjectName("EthernetSwitchPreferencesPageWidget")
        EthernetSwitchPreferencesPageWidget.resize(546, 455)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(EthernetSwitchPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(EthernetSwitchPreferencesPageWidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.uiEthernetSwitchesTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetSwitchesTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiEthernetSwitchesTreeWidget.setSizePolicy(sizePolicy)
        self.uiEthernetSwitchesTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiEthernetSwitchesTreeWidget.setFont(font)
        self.uiEthernetSwitchesTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiEthernetSwitchesTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiEthernetSwitchesTreeWidget.setRootIsDecorated(False)
        self.uiEthernetSwitchesTreeWidget.setObjectName("uiEthernetSwitchesTreeWidget")
        self.uiEthernetSwitchesTreeWidget.headerItem().setText(0, "1")
        self.uiEthernetSwitchesTreeWidget.header().setVisible(False)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiEthernetSwitchInfoTreeWidget = QtWidgets.QTreeWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetSwitchInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiEthernetSwitchInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiEthernetSwitchInfoTreeWidget.setIndentation(10)
        self.uiEthernetSwitchInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiEthernetSwitchInfoTreeWidget.setObjectName("uiEthernetSwitchInfoTreeWidget")
        self.uiEthernetSwitchInfoTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiEthernetSwitchInfoTreeWidget)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiNewEthernetSwitchPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiNewEthernetSwitchPushButton.setObjectName("uiNewEthernetSwitchPushButton")
        self.horizontalLayout_5.addWidget(self.uiNewEthernetSwitchPushButton)
        self.uiEditEthernetSwitchPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiEditEthernetSwitchPushButton.setEnabled(False)
        self.uiEditEthernetSwitchPushButton.setObjectName("uiEditEthernetSwitchPushButton")
        self.horizontalLayout_5.addWidget(self.uiEditEthernetSwitchPushButton)
        self.uiDeleteEthernetSwitchPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiDeleteEthernetSwitchPushButton.setEnabled(False)
        self.uiDeleteEthernetSwitchPushButton.setObjectName("uiDeleteEthernetSwitchPushButton")
        self.horizontalLayout_5.addWidget(self.uiDeleteEthernetSwitchPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(EthernetSwitchPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(EthernetSwitchPreferencesPageWidget)
        EthernetSwitchPreferencesPageWidget.setTabOrder(self.uiNewEthernetSwitchPushButton, self.uiDeleteEthernetSwitchPushButton)

    def retranslateUi(self, EthernetSwitchPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        EthernetSwitchPreferencesPageWidget.setWindowTitle(_translate("EthernetSwitchPreferencesPageWidget", "Ethernet switches"))
        EthernetSwitchPreferencesPageWidget.setAccessibleName(_translate("EthernetSwitchPreferencesPageWidget", "Ethernet switch templates"))
        self.uiEthernetSwitchInfoTreeWidget.headerItem().setText(0, _translate("EthernetSwitchPreferencesPageWidget", "1"))
        self.uiEthernetSwitchInfoTreeWidget.headerItem().setText(1, _translate("EthernetSwitchPreferencesPageWidget", "2"))
        self.uiNewEthernetSwitchPushButton.setText(_translate("EthernetSwitchPreferencesPageWidget", "&New"))
        self.uiEditEthernetSwitchPushButton.setText(_translate("EthernetSwitchPreferencesPageWidget", "&Edit"))
        self.uiDeleteEthernetSwitchPushButton.setText(_translate("EthernetSwitchPreferencesPageWidget", "&Delete"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/custom_adapters_configuration_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CustomAdaptersConfigurationDialog(object):
    def setupUi(self, CustomAdaptersConfigurationDialog):
        CustomAdaptersConfigurationDialog.setObjectName("CustomAdaptersConfigurationDialog")
        CustomAdaptersConfigurationDialog.resize(718, 450)
        CustomAdaptersConfigurationDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(CustomAdaptersConfigurationDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiAdaptersTreeWidget = QtWidgets.QTreeWidget(CustomAdaptersConfigurationDialog)
        self.uiAdaptersTreeWidget.setObjectName("uiAdaptersTreeWidget")
        self.verticalLayout.addWidget(self.uiAdaptersTreeWidget)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(CustomAdaptersConfigurationDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok|QtWidgets.QDialogButtonBox.Reset)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.verticalLayout.addWidget(self.uiButtonBox)

        self.retranslateUi(CustomAdaptersConfigurationDialog)
        self.uiButtonBox.accepted.connect(CustomAdaptersConfigurationDialog.accept)
        self.uiButtonBox.rejected.connect(CustomAdaptersConfigurationDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CustomAdaptersConfigurationDialog)

    def retranslateUi(self, CustomAdaptersConfigurationDialog):
        _translate = QtCore.QCoreApplication.translate
        CustomAdaptersConfigurationDialog.setWindowTitle(_translate("CustomAdaptersConfigurationDialog", "Custom adapters configuration"))
        self.uiAdaptersTreeWidget.headerItem().setText(0, _translate("CustomAdaptersConfigurationDialog", "Adapter number"))
        self.uiAdaptersTreeWidget.headerItem().setText(1, _translate("CustomAdaptersConfigurationDialog", "Port name"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for IOU preferences.
"""

import os
import copy

from gns3.qt import QtCore, QtWidgets, qslot, sip_is_deleted
from gns3.controller import Controller

from ..ui.iou_preferences_page_ui import Ui_IOUPreferencesPageWidget
from ..settings import IOU_SETTINGS

import logging
log = logging.getLogger(__name__)


class IOUPreferencesPage(QtWidgets.QWidget, Ui_IOUPreferencesPageWidget):

    """
    QWidget preference page for IOU.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._old_settings = None

        # connect signals
        self.uiIOURCPathToolButton.clicked.connect(self._iourcPathBrowserSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        Controller.instance().connected_signal.connect(self.loadPreferences)

    def _iourcPathBrowserSlot(self):
        """
        Slot to open a file browser and select an iourc file
        """

        documents_path = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DocumentsLocation)
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select the IOURC file", documents_path)
        if not path:
            return

        try:
            with open(path) as f:
                content = f.read().strip()
        except OSError as e:
            QtWidgets.QMessageBox.critical(self, "IOURC file", "{} cannot be read.\n{}".format(os.path.basename(path), str(e)))
            return
        except UnicodeDecodeError as e:
            QtWidgets.QMessageBox.critical(self, "IOURC file", "{} is not a licence file.".format(os.path.basename(path)))
            return
        if not content.startswith("[license]"):
            QtWidgets.QMessageBox.critical(self, "IOURC file", "This is not a valid IOURC file")
            return

        self.IOULicenceTextEdit.setPlainText(content)

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(IOU_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiLicensecheckBox.setEnabled(True)
        else:
            self.uiLicensecheckBox.setEnabled(False)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: IOU settings
        """

        if settings["iourc_content"]:
            self.IOULicenceTextEdit.blockSignals(True)
            self.IOULicenceTextEdit.setPlainText(settings["iourc_content"])
            self.IOULicenceTextEdit.blockSignals(False)
        self.uiLicensecheckBox.setChecked(settings["license_check"])

    def loadPreferences(self):
        """
        Loads IOU preferences.
        """

        if Controller.instance().connected():
            Controller.instance().get("/iou_license", self._getSettingsCallback)
        else:
            log.error("Cannot load the IOU license in the preferences dialog: not connected to the controller")

    @qslot
    def _getSettingsCallback(self, result, error=False, **kwargs):

        if sip_is_deleted(self):
            return
        if error:
            if "message" in result:
                log.error("Error while getting the IOU license information: {}".format(result["message"]))
            return
        self._old_settings = copy.copy(result)
        self._populateWidgets(result)

    def savePreferences(self):
        """
        Saves IOU preferences.
        """

        if not self._old_settings:
            return

        iourc_content = self.IOULicenceTextEdit.toPlainText().strip().replace("\r\n", "\n")

        new_settings = {"iourc_content": iourc_content,
                        "license_check": self.uiLicensecheckBox.isChecked()}

        if self._old_settings != new_settings:
            Controller.instance().put("/iou_license", self._saveSettingsCallback, new_settings, timeout=60)
            self._old_settings = copy.copy(new_settings)

    def _saveSettingsCallback(self, result, error=False, **kwargs):
        if error and "message" in result:
            QtWidgets.QMessageBox.critical(self, "Save settings", "Error while saving settings: {}".format(result["message"]))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/packet_capture_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PacketCapturePreferencesPageWidget(object):
    def setupUi(self, PacketCapturePreferencesPageWidget):
        PacketCapturePreferencesPageWidget.setObjectName("PacketCapturePreferencesPageWidget")
        PacketCapturePreferencesPageWidget.resize(839, 444)
        self.verticalLayout = QtWidgets.QVBoxLayout(PacketCapturePreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiSettingsGroupBox = QtWidgets.QGroupBox(PacketCapturePreferencesPageWidget)
        self.uiSettingsGroupBox.setObjectName("uiSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiPreconfiguredCaptureReaderCommandLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiPreconfiguredCaptureReaderCommandLabel.setObjectName("uiPreconfiguredCaptureReaderCommandLabel")
        self.gridLayout.addWidget(self.uiPreconfiguredCaptureReaderCommandLabel, 0, 0, 1, 2)
        self.uiPreconfiguredCaptureReaderCommandComboBox = QtWidgets.QComboBox(self.uiSettingsGroupBox)
        self.uiPreconfiguredCaptureReaderCommandComboBox.setObjectName("uiPreconfiguredCaptureReaderCommandComboBox")
        self.gridLayout.addWidget(self.uiPreconfiguredCaptureReaderCommandComboBox, 1, 0, 1, 1)
        self.uiPreconfiguredCaptureReaderCommandPushButton = QtWidgets.QPushButton(self.uiSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPreconfiguredCaptureReaderCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiPreconfiguredCaptureReaderCommandPushButton.setSizePolicy(sizePolicy)
        self.uiPreconfiguredCaptureReaderCommandPushButton.setObjectName("uiPreconfiguredCaptureReaderCommandPushButton")
        self.gridLayout.addWidget(self.uiPreconfiguredCaptureReaderCommandPushButton, 1, 1, 1, 1)
        self.uiCaptureReaderCommandLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiCaptureReaderCommandLabel.setEnabled(True)
        self.uiCaptureReaderCommandLabel.setObjectName("uiCaptureReaderCommandLabel")
        self.gridLayout.addWidget(self.uiCaptureReaderCommandLabel, 2, 0, 1, 1)
        self.uiAutoStartCheckBox = QtWidgets.QCheckBox(self.uiSettingsGroupBox)
        self.uiAutoStartCheckBox.setEnabled(True)
        self.uiAutoStartCheckBox.setChecked(False)
        self.uiAutoStartCheckBox.setObjectName("uiAutoStartCheckBox")
        self.gridLayout.addWidget(self.uiAutoStartCheckBox, 4, 0, 1, 2)
        self.uiCaptureAnalyzerCommandLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiCaptureAnalyzerCommandLabel.setObjectName("uiCaptureAnalyzerCommandLabel")
        self.gridLayout.addWidget(self.uiCaptureAnalyzerCommandLabel, 5, 0, 1, 1)
        self.uiCaptureAnalyzerCommandLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiCaptureAnalyzerCommandLineEdit.setObjectName("uiCaptureAnalyzerCommandLineEdit")
        self.gridLayout.addWidget(self.uiCaptureAnalyzerCommandLineEdit, 6, 0, 1, 2)
        self.uiCaptureReaderCommandLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiCaptureReaderCommandLineEdit.setObjectName("uiCaptureReaderCommandLineEdit")
        self.gridLayout.addWidget(self.uiCaptureReaderCommandLineEdit, 3, 0, 1, 2)
        self.verticalLayout.addWidget(self.uiSettingsGroupBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(250, 17, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(PacketCapturePreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(PacketCapturePreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(PacketCapturePreferencesPageWidget)

    def retranslateUi(self, PacketCapturePreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        PacketCapturePreferencesPageWidget.setWindowTitle(_translate("PacketCapturePreferencesPageWidget", "Packet capture"))
        self.uiSettingsGroupBox.setTitle(_translate("PacketCapturePreferencesPageWidget", "Settings"))
        self.uiPreconfiguredCaptureReaderCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Preconfigured packet capture reader commands:"))
        self.uiPreconfiguredCaptureReaderCommandPushButton.setText(_translate("PacketCapturePreferencesPageWidget", "&Set"))
        self.uiCaptureReaderCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Packet capture reader command:"))
        self.uiAutoStartCheckBox.setText(_translate("PacketCapturePreferencesPageWidget", "Automatically start the packet capture application"))
        self.uiCaptureAnalyzerCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Packet capture analyzer command:"))
        self.uiCaptureReaderCommandLineEdit.setToolTip(_translate("PacketCapturePreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><p>%c = capture file (PCAP format)</p></body></html>"))
        self.uiRestoreDefaultsPushButton.setText(_translate("PacketCapturePreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import pathlib
import ipaddress
import urllib.request
import urllib.parse
import copy
import uuid

from typing import List, Union, Callable, Optional
from .qt import sip
from .version import __version__, __version_info__
from .qt import QtCore, QtNetwork, QtWidgets, QtWebSockets, qpartial
from .utils import parse_version
from .dialogs.login_dialog import LoginDialog

from .http_client_error import (
    HttpClientError,
    HttpClientCancelledRequestError,
    HttpClientBadRequestError,
    HttpClientUnauthorizedError,
    HttpClientTimeoutError
)

import logging
log = logging.getLogger(__name__)


class QNetworkReplyWatcher(QtCore.QObject):
    """
    Synchronously wait for a QNetworkReply to be completed
    """

    def __init__(
            self,
            show_progress: bool = True,
            progress_text: str = None,
            parent: QtWidgets.QWidget = None
    ):

        if not parent:
            from gns3.main_window import MainWindow
            parent = MainWindow.instance()

        super().__init__(parent)

        if show_progress:
            if not progress_text:
                progress_text = "Waiting for controller..."
            self._progress = QtWidgets.QProgressDialog(progress_text, "Cancel", 0, 0, parent)
            self._progress.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
            self._progress.setWindowModality(QtCore.Qt.ApplicationModal)
        else:
            self._progress = None

    def _updateProgress(self, bytes_sent: int, bytes_total: int) -> None:
        """
        Update upload or download progress.
        """

        if bytes_total > 0:
            if not self._progress.maximum():
                self._progress.setMaximum(100)
            self._progress.setValue(int(100 * bytes_sent / bytes_total))

    def waitForReply(self, reply: QtNetwork.QNetworkReply, uploading: bool = False, timeout=60) -> None:
        """
        Wait for the QNetworkReply to be complete or for the timeout

        :param reply: QNetworkReply instance
        :param uploading: Whether the reply is uploading or not
        :param timeout: Number of seconds before timeout
        """

        loop = QtCore.QEventLoop()

        if timeout:
            timer = QtCore.QTimer(self)
            timer.setSingleShot(True)
            timer.timeout.connect(lambda: loop.exit(1))
            timer.start(timeout * 1000)

        reply.finished.connect(loop.quit)

        if self._progress:
            reply.finished.connect(self._progress.close)
            if uploading:
                reply.uploadProgress.connect(self._updateProgress)
            reply.downloadProgress.connect(self._updateProgress)
            self._progress.canceled.connect(reply.abort)
            self._progress.show()

        if not loop.isRunning():
            if loop.exec_() == 1:
                raise HttpClientTimeoutError(
                    f"Request to '{reply.url().toString()}' timed out after {timeout} seconds")

        if timeout and timer.isActive():
            timer.stop()


class HTTPClient(QtCore.QObject):
    """
    HTTP client to communicate with the controller REST API

    :param settings: Dictionary with connection information to the server
    :param max_retry_connection: Number of time to try to connect to the server
    :param prefix: API version (prefix)
    """

    connected_signal = QtCore.Signal()
    disconnected_signal = QtCore.Signal()

    def __init__(self, settings: dict, max_retry_connection: int = 5, prefix: str = "/v3"):

        super().__init__()
        from gns3.main_window import MainWindow
        self._main_window = MainWindow.instance()

        self._protocol = settings.get("protocol", "http")
        self._host = settings["host"]
        self._prefix = prefix
        self._auth_attempted = False
        self._jwt_token = None
        try:
            if self._host is None or self._host == "0.0.0.0":
                self._host = "127.0.0.1"
            elif ":" in self._host and ipaddress.IPv6Address(self._host) and str(ipaddress.IPv6Address(self._host)) == "::":
                self._host = "::1"
        except ipaddress.AddressValueError:
            log.error("Invalid host name %s", self._host)

        self._port = int(settings["port"])
        self._username = settings.get("username")
        self._password = settings.get("password")
        self._retry = 0  # how many times we have already retried connection
        self._max_retry_connection = max_retry_connection
        self._retry_connection = False
        self._connected = False
        self._shutdown = False  # shutdown in progress
        self._accept_insecure_certificate = settings.get("accept_insecure_certificate", False)

        # Add custom CA
        # ssl_config = QtNetwork.QSslConfiguration.defaultConfiguration()
        # if ssl_config.addCaCertificates("/path/to/rootCA.crt"):
        #     log.debug("CA certificate added")
        # QtNetwork.QSslConfiguration.setDefaultConfiguration(ssl_config)

        if self._protocol == "https":
            if not QtNetwork.QSslSocket.supportsSsl():
                log.error("SSL is not supported")
            else:
                log.debug(f"SSL is supported, version: {QtNetwork.QSslSocket().sslLibraryBuildVersionString()}")

        self._network_manager = QtNetwork.QNetworkAccessManager()

        # Allow to follow redirections
        #self._network_manager.setRedirectPolicy(QtNetwork.QNetworkRequest.SameOriginRedirectPolicy)

        # Buffer for JSON notifications
        self._buffer = {}

        # List of query waiting for the connection
        self._query_waiting_connections = []

        # To catch SSL errors
        self._network_manager.sslErrors.connect(self.handleSslError)

        # Store SSL error exceptions
        self._ssl_exceptions = {}

    def host(self) -> str:
        """
        Return the server IP or hostname
        """

        return self._host

    def setHost(self, host: str) -> None:
        """
        Set the server IP or hostname
        """

        self._host = host

    def port(self) -> int:
        """
        Return the server port
        """

        return self._port

    def setPort(self, port: int) -> None:
        """
        Set the server port
        """

        self._port = port

    def protocol(self) -> str:
        """
        Return the transport protocol (HTTP, HTTPS)
        """

        return self._protocol

    def setAcceptInsecureCertificate(self, certificate: bool) -> None:
        """
        Does the server accept this insecure SSL certificate digest
        """

        self._accept_insecure_certificate = certificate

    def url(self) -> str:
        """
        Returns current server url
        """

        if ":" in self.host():
            return "{}://[{}]:{}".format(self.protocol(), self.host(), self.port())
        return "{}://{}:{}".format(self.protocol(), self.host(), self.port())

    def fullUrl(self) -> str:
        """
        Returns current server URL
        """

        host = self.host()
        if ":" in self.host():
            host = "[{}]".format(host)

        return "{}://{}:{}".format(self.protocol(), host, self.port())

    def shutdown(self) -> None:
        """
        Stop the server and stop to accept queries
        """

        self._shutdown = True
        self.sendRequest("POST", "/shutdown", callback=None, wait=False)

    def connected(self) -> bool:
        """
        Return whether the client is connected or not
        """

        return self._connected

    def close(self) -> None:
        """
        Closes the connection with the server
        """

        self._connected = False

    def _request(self, url: str) -> QtNetwork.QNetworkRequest:
        """
        Return a QNetworkRequest object. You can mock this
        if you want low level mocking.

        :param url: URL for the request
        :returns: QtNetwork.QNetworkRequest instance
        """

        return QtNetwork.QNetworkRequest(QtCore.QUrl(url))

    def sendRequest(
            self,
            method: str,
            endpoint: str,
            callback: Callable = None,
            body: Union[dict, pathlib.Path, str] = None,
            context: dict = None,
            download_progress_callback: Callable = None,
            show_progress: bool = True,
            progress_text: str = None,
            disconnect_on_error: bool = False,
            timeout: int = 120,
            params: dict = None,
            raw: bool = False,
            wait: bool = False
    ) -> Optional[Union[str, bytes]]:
        """
        Send a request to the server

        :param method: HTTP method
        :param endpoint: API endpoint
        :param callback: callback method to call when the server replies
        :param body: Body to send (dictionary, string or pathlib.Path)
        :param context: Pass a context to the response callback
        :param download_progress_callback: Callback called when received something, it can be an incomplete reply
        :param show_progress: Display a progress bar dialog
        :param progress_text: Custom text to display in the progress bar dialog
        :param disconnect_on_error: Disconnect from server if there is an error
        :param timeout: Delay in seconds before raising a timeout
        :param params: Query parameters
        :param raw: Return the raw server reply body (bytes)
        :param wait: Wait for server reply asynchronously
        """

        # Shutdown in progress do not execute the request
        if self._shutdown:
            return

        request = qpartial(
            self._executeHTTPQuery,
            method,
            endpoint,
            qpartial(callback),
            body,
            context=context,
            download_progress_callback=download_progress_callback,
            show_progress=show_progress,
            disconnect_on_error=disconnect_on_error,
            progress_text=progress_text,
            timeout=timeout,
            params=params,
            raw=raw,
            wait=wait
        )

        if self._connected:
            return request()
        else:
            self._query_waiting_connections.append((request, callback))
            # enqueue the first query and open the connection if we are not connected
            if len(self._query_waiting_connections) == 1:
                log.debug("Connection to {}".format(self.url()))
                self.connectToServer()

    def _addBodyToRequest(
            self,
            body: Optional[Union[dict, pathlib.Path, str]],
            request: QtNetwork.QNetworkRequest
    ) -> Optional[Union[QtCore.QBuffer, QtCore.QFile]]:
        """
        Add the required headers for sending the body.

        :param body: body to send in request
        :returns: body compatible with Qt
        """

        if body is None:
            return None

        if isinstance(body, dict):
            body = json.dumps(body)
            data = QtCore.QByteArray(body.encode())
            body = QtCore.QBuffer(self)
            body.setData(data)
            body.open(QtCore.QIODevice.ReadOnly)
            request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/json")
            request.setHeader(QtNetwork.QNetworkRequest.ContentLengthHeader, str(data.size()))
            #request.setRawHeader(b"Content-Length", str(data.size()).encode())
            return body
        elif isinstance(body, pathlib.Path):
            body = QtCore.QFile(str(body), self)
            body.open(QtCore.QFile.ReadOnly)
            request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/octet-stream")
            request.setHeader(QtNetwork.QNetworkRequest.ContentLengthHeader, str(body.size()))
            #request.setRawHeader(b"Content-Length", str(body.size()).encode())
            return body
        elif isinstance(body, str):
            data = QtCore.QByteArray(body.encode())
            body = QtCore.QBuffer(self)
            body.setData(data)
            body.open(QtCore.QIODevice.ReadOnly)
            request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/octet-stream")
            #request.setRawHeader(b"Content-Length", str(data.size()).encode())
            return body
        else:
            return None

    def _addAuth(self, request: QtNetwork.QNetworkRequest) -> QtNetwork.QNetworkRequest:
        """
        Add the JWT token in the authentication header
        """

        if self._jwt_token:
            request.setRawHeader(b"Authorization", f"Bearer {self._jwt_token}".encode())
        return request

    def connectWebSocket(self, websocket: QtWebSockets.QWebSocket, endpoint: str) -> QtWebSockets.QWebSocket:
        """
        Connect to websocket endpoint
        """

        ws_protocol = "ws"
        if self._protocol == "https":
            ws_protocol = "wss"
        host = self._getHostForQuery()
        port = self._port
        prefix = self._prefix
        jwt_token = self._jwt_token
        request = websocket.request()
        ws_url = f"{ws_protocol}://{host}:{port}{prefix}{endpoint}?token={jwt_token}"
        log.debug(f"Connecting to WebSocket: {ws_url}")
        request.setUrl(QtCore.QUrl(ws_url))
        websocket.open(request)
        return websocket

    def _getHostForQuery(self) -> str:
        """
        Get a hostname that can be used by Qt
        """

        try:
            ip = self._host.rsplit('%', 1)[0]
            ipaddress.IPv6Address(ip)  # remove any scope ID
            # this is an IPv6 address, we must surround it with brackets to be used with QUrl.
            host = "[{}]".format(ip)
        except ipaddress.AddressValueError:
            host = self._host
        return host

    def _paramsToQueryString(self, params: dict) -> str:
        """
        :param params: Dictionary of query string parameters
        :returns: String of the query string
        """

        if not params:
            query_string = ""
        else:
            query_string = "?"
            params = params.copy()
            for key, value in params.copy().items():
                if value is None:
                    del params[key]
            query_string += urllib.parse.urlencode(params)
        return query_string

    def _dataReadySlot(self, reply, callback, context):
        """
        Process a packet received on the notification feed.
        The feed can contain qpartial JSON. If we find fragmented
        JSON, we keep it for the next packet
        """

        if reply.error() != QtNetwork.QNetworkReply.NoError:
            return

        # HTTP error
        status = reply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
        if status >= 300:
            return

        content = bytes(reply.readAll())
        if not content:
            return
        content_type = reply.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
        if content_type == "application/json":
            content = content.decode("utf-8")
            if context["query_id"] in self._buffer:
                content = self._buffer[context["query_id"]] + content
            try:
                while True:
                    content = content.lstrip(" \r\n\t")
                    answer, index = json.JSONDecoder().raw_decode(content)
                    callback(answer, context=context)
                    content = content[index:]
            except ValueError:  # Partial JSON
                self._buffer[context["query_id"]] = content
        else:
            callback(content, context=context)

    def _timeoutSlot(self, reply: QtNetwork.QNetworkReply, timeout: int) -> None:
        """
        Call for all requests, you need to check the status of the response
        """

        # We check if we received HTTP headers
        if not sip.isdeleted(reply) and reply.isRunning() and not len(reply.rawHeaderList()) > 0:
            if not reply.error() != QtNetwork.QNetworkReply.NoError:
                log.warning(f"Timeout after {timeout} seconds for request {reply.url().toString()}. \
                Please check the connection is not blocked by a firewall or an anti-virus.")
                reply.abort()

    def disconnect(self) -> None:
        """
        Disconnect from the server
        """

        if self.connected():
            self.disconnected_signal.emit()
            self.close()

    def _requestCredentialsFromUser(self):
        """
        Request credentials from user

        :return: username, password
        """

        username = password = None
        login_dialog = LoginDialog(self._main_window)
        if self._username:
            login_dialog.setUsername(self._username)
        login_dialog.show()
        login_dialog.raise_()
        if login_dialog.exec_():
            username = login_dialog.getUsername()
            password = login_dialog.getPassword()
        return username, password

    def _handleUnauthorizedRequest(self, reply: QtNetwork.QNetworkReply) -> None:
        """
        Request the username / password to authenticate with the server
        """

        if not self._username or not self._password or self._auth_attempted is True:
            username, password = self._requestCredentialsFromUser()
        else:
            username = self._username
            password = self._password

        if username and password:
            body = {
                "username": username,
                "password": password
            }
            self._auth_attempted = True
            content = self._executeHTTPQuery("POST", "/access/users/authenticate", body=body, wait=True)
            if content:
                log.info(f"Authenticated with controller {self._host} on port {self._port}")
                token = content.get("access_token")
                if token:
                    self._auth_attempted = False
                    self._jwt_token = token
                    return
        else:
            raise HttpClientUnauthorizedError(f"{reply.errorString()}")

    def _validateServerVersion(self, content: dict) -> None:
        """
        Validate the server version.
        """

        version = content.get("version")
        local = content.get("local")

        if version is None or local is None:
            raise HttpClientBadRequestError(f"The server is not a GNS3 server: {content}")

        if version.split("+")[0] != __version__.split("+")[0]:
            msg = f"Client version {__version__} is not the same as server version {version}"
            # We don't allow different major version to interact even with dev build
            if __version_info__[3] == 0 or parse_version(__version__)[:2] != parse_version(version)[:2]:
                raise HttpClientError(msg)
            log.warning(f"{msg}\nUsing different versions may result in unexpected problems.\n"
                        "Please upgrade or use at your own risk.")

    def _retryConnectionToServer(self, wait_time: int = 5) -> bool:
        """
        Retry a connection.
        """

        self._retry += 1
        loop = QtCore.QEventLoop()
        progress = QtWidgets.QProgressDialog(
            f"Retrying connection to controller (#{self._retry}/{self._max_retry_connection})",
            "Cancel",
            0,
            0,
            self._main_window
        )
        progress.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        progress.setWindowModality(QtCore.Qt.ApplicationModal)
        progress.canceled.connect(loop.quit)
        progress.show()
        timer = QtCore.QTimer(self)
        timer.setSingleShot(True)
        timer.timeout.connect(loop.quit)
        timer.start(wait_time * 1000)
        loop.exec_()
        if progress.wasCanceled():
            return False
        progress.close()
        return self.connectToServer()

    def checkServerRunning(self) -> bool:
        """
        Check if server is running.
        """

        try:
            content = self._executeHTTPQuery("GET", "/version", wait=True, show_progress=False, timeout=2)
            self._validateServerVersion(content)
        except HttpClientError:
            return False
        return True

    def connectToServer(self) -> bool:
        """
        Connect to the GNS3 server
        """

        try:
            self._retry_connection = True
            content = self._executeHTTPQuery("GET", "/version", wait=True)
            self._validateServerVersion(content)
            self._retry_connection = False
            self._executeHTTPQuery("GET", "/access/users/me", wait=True)
        except HttpClientCancelledRequestError:
            return False  # operation cancelled by user
        except HttpClientError as e:
            if self._retry_connection and self._retry < self._max_retry_connection:
                return self._retryConnectionToServer()
            for request, callback in self._query_waiting_connections:
                if callback is not None:
                    callback({"message": str(e)}, error=True)
            self._query_waiting_connections = []
            self.disconnect()
            error_dialog = QtWidgets.QMessageBox(self._main_window)
            error_dialog.setWindowModality(QtCore.Qt.ApplicationModal)
            error_dialog.setWindowTitle("Connecting to server")
            error_dialog.setText(f"Error while connecting to the server")
            error_dialog.setDetailedText(f"{e}")
            error_dialog.setIcon(QtWidgets.QMessageBox.Critical)
            error_dialog.show()
        else:
            self._connected = True
            self._retry = 0
            self.connected_signal.emit()
            for request, callback in self._query_waiting_connections:
                if request:
                    request()
            self._query_waiting_connections = []
            return True
        return False

    def _prepareRequest(self, method: str, endpoint: str, params: dict) -> QtNetwork.QNetworkRequest:
        """
        Return a QtNetwork.QNetworkRequest() instance
        """

        protocol = self._protocol
        host = self._getHostForQuery()
        port = self._port
        prefix = self._prefix
        query_string = self._paramsToQueryString(params)

        log.debug(f"{method} {protocol}://{host}:{port}{prefix}{endpoint}{query_string}")
        url = QtCore.QUrl(f"{protocol}://{host}:{port}{prefix}{endpoint}{query_string}")

        request = self._request(url)
        request = self._addAuth(request)
        request.setHeader(QtNetwork.QNetworkRequest.UserAgentHeader, f"GNS3 QT Client v{__version__}")
        return request

    def _executeHTTPQuery(
            self,
            method: str,
            endpoint: str,
            callback: Callable = None,
            body: Union[dict, pathlib.Path, str] = None,
            context: dict = None,
            download_progress_callback: Callable = None,
            show_progress: bool = True,
            progress_text: str = None,
            disconnect_on_error: bool = False,
            timeout: int = 60,
            params: dict = None,
            raw: bool = False,
            wait: bool = False,

    ) -> Optional[Union[str, bytes]]:
        """
        Send an HTTP request

        :param method: HTTP method
        :param endpoint: API endpoint
        :param callback: callback method to call when the server replies
        :param body: Body to send (dictionary, string or pathlib.Path)
        :param context: Pass a context to the response callback
        :param download_progress_callback: Callback called when received something, it can be an incomplete reply
        :param show_progress: Display a progress bar dialog
        :param progress_text: Custom text to display in the progress bar dialog
        :param disconnect_on_error: Disconnect from server if there is an error
        :param timeout: Delay in seconds before raising a timeout
        :param params: Query parameters
        :param raw: Return the raw server reply body (bytes)
        :param wait: Wait for server reply asynchronously
        """

        request = self._prepareRequest(method, endpoint, params)

        # By default Qt doesn't support GET with body
        # even if it's in the RFC that's why we need to use sendCustomRequest
        body = self._addBodyToRequest(body, request)

        try:
            reply = self._network_manager.sendCustomRequest(request, method.encode(), body)
        except SystemError as e:
            log.error("Can't send query: {}".format(str(e)))
            return

        if context:
            context = copy.copy(context)
            context["query_id"] = str(uuid.uuid4())

        if download_progress_callback is not None:
            reply.readyRead.connect(qpartial(self._dataReadySlot, reply, download_progress_callback, context))

        if wait:
            uploading = False
            if request.header(QtNetwork.QNetworkRequest.ContentTypeHeader) == "application/octet-stream":
                uploading = True

            QNetworkReplyWatcher(show_progress, progress_text).waitForReply(reply, uploading, timeout)
            try:
                content = self._processReply(reply, disconnect_on_error, raw)
                if callback:
                    callback(content, context=context)
                else:
                    return content
            except HttpClientError as e:
                if callback:
                    callback({"message": str(e)}, error=True, context=context)
                else:
                    raise
        else:
            reply.finished.connect(
                qpartial(self._processAsyncReply, reply, callback, body, context, timeout, disconnect_on_error, raw)
            )

            if timeout is not None:
                QtCore.QTimer.singleShot(timeout * 1000, qpartial(self._timeoutSlot, reply, timeout))

    def _processAsyncReply(
            self,
            reply: QtNetwork.QNetworkReply,
            callback: Callable,
            body: Union[dict, pathlib.Path, str] = None,
            context: dict = None,
            timeout: int = None,
            disconnect_on_error: bool = False,
            raw: bool = False
    ) -> None:

        if body is not None:
            body.close()

        try:
            content = self._processReply(reply, disconnect_on_error, raw)
            if callback:
                callback(content, context=context)
            if timeout is not None:
                QtCore.QTimer.singleShot(timeout * 1000, qpartial(self._timeoutSlot, reply, timeout))
        except HttpClientCancelledRequestError:
            return  # operation cancelled by user
        except HttpClientError as e:
            if callback:
                callback({"message": str(e)}, error=True, context=context)
            else:
                # display error because no callback is configured to handle it
                log.error(f"{e}")
        finally:
            reply.deleteLater()

    def _processReply(
            self,
            reply: QtNetwork.QNetworkReply,
            disconnect_on_error: bool = False,
            raw: bool = False
    ) -> Optional[Union[str, bytes]]:
        """
        Process the information returned by QtNetwork.QNetworkReply
        """

        status = reply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
        content_type = reply.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
        if reply.error() == QtNetwork.QNetworkReply.NoError:
            try:
                content = bytes(reply.readAll())
                if raw is False:
                    content = content.decode("utf-8").strip(" \0\n\t")
                    if content and content_type == "application/json":
                        content = json.loads(content)
            except ValueError as e:
                raise HttpClientBadRequestError(f"Could not read data with content type '{content_type}' returned from"
                                                f" '{reply.url().toString()}': {e} (raw={raw})")
            if status >= 400:
                raise HttpClientError(f"Request to '{reply.url().toString()}' has returned HTTP code {status}")
            return content
        elif reply.error() == QtNetwork.QNetworkReply.NetworkSessionFailedError:
            return  # ignore the network session failed error to let the network manager recover from it
        elif reply.error() == QtNetwork.QNetworkReply.OperationCanceledError:
            raise HttpClientCancelledRequestError("Request cancelled")
        else:
            error_message = f"{reply.errorString()} (HTTP code {status})"
            if content_type == "application/json":
                try:
                    custom_error_message = json.loads(bytes(reply.readAll()).decode("utf-8")).get("message")
                    if custom_error_message:
                        error_message = custom_error_message
                except ValueError as e:
                    log.debug(f"Could not read server error message: {e}")
            log.debug(error_message)
            if status == 401 and reply.rawHeader(b"WWW-Authenticate") == b"Bearer":
                self._handleUnauthorizedRequest(reply)
            else:
                if disconnect_on_error:
                    self.disconnect()
                raise HttpClientError(error_message)

    def handleSslError(self, reply: QtNetwork.QNetworkReply, ssl_errors: List[QtNetwork.QSslError]) -> None:
        """
        Handle SSL errors
        """

        if self._accept_insecure_certificate:
            reply.ignoreSslErrors()
            return

        url = reply.request().url()
        host_port_key = f"{url.host()}:{url.port()}"

        # get the certificate digest
        ssl_config = reply.sslConfiguration()
        peer_cert = ssl_config.peerCertificate()
        digest = peer_cert.digest()

        if host_port_key in self._ssl_exceptions:

            if self._ssl_exceptions[host_port_key] == digest:
                reply.ignoreSslErrors()
                return

        msgbox = QtWidgets.QMessageBox(self._main_window)
        msgbox.setWindowTitle("SSL error detected")
        msgbox.setText(f"This server could not prove that it is {url.host()}:{url.port()}. Please carefully examine the certificate to make sure the server can be trusted.")
        msgbox.setInformativeText(f"{ssl_errors[0].errorString()}")
        msgbox.setDetailedText(peer_cert.toText())
        msgbox.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        connect_button = QtWidgets.QPushButton(f"&Connect to {url.host()}:{url.port()}", msgbox)
        msgbox.addButton(connect_button, QtWidgets.QMessageBox.YesRole)
        abort_button = QtWidgets.QPushButton("&Abort", msgbox)
        msgbox.addButton(abort_button, QtWidgets.QMessageBox.RejectRole)
        msgbox.setDefaultButton(abort_button)
        msgbox.setIcon(QtWidgets.QMessageBox.Critical)
        msgbox.exec_()

        if msgbox.clickedButton() == connect_button:
            self._ssl_exceptions[host_port_key] = digest
            reply.ignoreSslErrors()
        else:
            for error in ssl_errors:
                log.error(f"SSL error detected: {error.errorString()}")
            self._main_window.close()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shlex
import pytest

from unittest.mock import patch
from gns3.spice_console import spiceConsole


def test_spice_console_on_linux_and_mac(vpcs_device):
    with patch('subprocess.Popen') as popen, \
            patch('sys.platform', new="linux"):

        vpcs_device.settings()["console_host"] = "localhost"
        spiceConsole(vpcs_device, '2525', 'command %h %p')
        popen.assert_called_once_with(shlex.split('command localhost 2525'), env=os.environ)


def test_spice_console_on_windows(vpcs_device):
    with patch('subprocess.Popen') as popen, \
            patch('sys.platform', new="win"):
        vpcs_device.settings()["console_host"] = "localhost"
        spiceConsole(vpcs_device, '2525', 'command %h %p')
        popen.assert_called_once_with('command localhost 2525')


# def test_spice_console_on_linux_with_popen_issues():
#     with patch('subprocess.Popen', side_effect=OSError("Dummy")), \
#             patch('sys.platform', new="linux"):
#
#         with pytest.raises(OSError):
#             spiceConsole('localhost', '2525', 'command %h %p')


def test_spice_console_with_ipv6_support(vpcs_device):
    with patch('subprocess.Popen') as popen, \
            patch('sys.platform', new="linux"):

        vpcs_device.settings()["console_host"] = "::1"
        spiceConsole(vpcs_device, '2525', 'command %h %p')
        popen.assert_called_once_with(shlex.split('command [::1] 2525'), env=os.environ)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Style editor to edit Shape items.
"""

from ..qt import QtCore, QtWidgets, QtGui
from ..ui.style_editor_dialog_ui import Ui_StyleEditorDialog
from ..items.shape_item import ShapeItem
from ..items.line_item import LineItem
from ..items.rectangle_item import RectangleItem


class StyleEditorDialog(QtWidgets.QDialog, Ui_StyleEditorDialog):

    """
    Style editor dialog.

    :param parent: parent widget
    :param items: list of items
    """

    def __init__(self, parent, items):

        super().__init__(parent)
        self.setupUi(self)

        self._items = items
        self.uiColorPushButton.clicked.connect(self._setColorSlot)
        self.uiBorderColorPushButton.clicked.connect(self._setBorderColorSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self._applyPreferencesSlot)

        self.uiBorderStyleComboBox.addItem("Solid", QtCore.Qt.SolidLine)
        self.uiBorderStyleComboBox.addItem("Dash", QtCore.Qt.DashLine)
        self.uiBorderStyleComboBox.addItem("Dot", QtCore.Qt.DotLine)
        self.uiBorderStyleComboBox.addItem("Dash Dot", QtCore.Qt.DashDotLine)
        self.uiBorderStyleComboBox.addItem("Dash Dot Dot", QtCore.Qt.DashDotDotLine)
        if True not in list(map(lambda item: isinstance(item, LineItem), items)):
            self.uiBorderStyleComboBox.addItem("No border", QtCore.Qt.NoPen)

        # use the first item in the list as the model
        first_item = items[0]
        pen = first_item.pen()
        if hasattr(first_item, "brush"):  # Line don't have brush
            brush = first_item.brush()
            self._color = brush.color()
            self.uiColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._color.red(),
                                                                                                  self._color.green(),
                                                                                                  self._color.blue(),
                                                                                                  self._color.alpha()))
        else:
            self.uiColorLabel.hide()
            self.uiColorPushButton.hide()
            self._color = None

        self._border_color = pen.color()
        self.uiBorderColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._border_color.red(),
                                                                                                    self._border_color.green(),
                                                                                                    self._border_color.blue(),
                                                                                                    self._border_color.alpha()))
        if isinstance(first_item, RectangleItem):
            # use the horizontal corner radius first and then the vertical one if it's not set
            # maybe we allow configuring them separately in the future
            corner_radius = first_item.horizontalCornerRadius()
            if not corner_radius:
                corner_radius = first_item.verticalCornerRadius()
            self.uiCornerRadiusSpinBox.setValue(corner_radius)
        self.uiRotationSpinBox.setValue(int(first_item.rotation()))
        self.uiBorderWidthSpinBox.setValue(pen.width())
        index = self.uiBorderStyleComboBox.findData(pen.style())
        if index != -1:
            self.uiBorderStyleComboBox.setCurrentIndex(index)

    def _setColorSlot(self):
        """
        Slot to select the filling color.
        """

        color = QtWidgets.QColorDialog.getColor(self._color, self, "Select Color", QtWidgets.QColorDialog.ShowAlphaChannel)
        if color.isValid():
            self._color = color
            self.uiColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._color.red(),
                                                                                                  self._color.green(),
                                                                                                  self._color.blue(),
                                                                                                  self._color.alpha()))

    def _setBorderColorSlot(self):
        """
        Slot to select the border color.
        """

        color = QtWidgets.QColorDialog.getColor(self._border_color, self, "Select Color", QtWidgets.QColorDialog.ShowAlphaChannel)
        if color.isValid():
            self._border_color = color
            self.uiBorderColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._border_color.red(),
                                                                                                        self._border_color.green(),
                                                                                                        self._border_color.blue(),
                                                                                                        self._border_color.alpha()))

    def _applyPreferencesSlot(self):
        """
        Applies the new style settings.
        """

        border_style = QtCore.Qt.PenStyle(self.uiBorderStyleComboBox.itemData(self.uiBorderStyleComboBox.currentIndex()))
        pen = QtGui.QPen(self._border_color, self.uiBorderWidthSpinBox.value(), border_style, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
        if self._color:
            brush = QtGui.QBrush(self._color)
        else:
            brush = None

        for item in self._items:
            item.setPen(pen)
            # on multi-selection it's possible to select many type of items
            # but brush can be applied only on ShapeItem,
            if brush and isinstance(item, ShapeItem):
                item.setBrush(brush)
            if isinstance(item, RectangleItem):
                corner_radius = self.uiCornerRadiusSpinBox.value()
                # use the corner radius for both horizontal (rx) and vertical (ry)
                # maybe we support setting them separately in the future
                item.setHorizontalCornerRadius(corner_radius)
                item.setVerticalCornerRadius(corner_radius)
            item.setRotation(self.uiRotationSpinBox.value())

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._applyPreferencesSlot()
        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.node import Node

import logging
log = logging.getLogger(__name__)


class FrameRelaySwitch(Node):

    """
    Frame-Relay switch.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    URL_PREFIX = "frame_relay_switch"

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        # this is an always-on node
        self.setStatus(Node.started)
        self._always_on = True
        self.settings().update({"mappings": {}})

    def info(self):
        """
        Returns information about this Frame Relay switch.

        :returns: formatted string
        """

        info = """Frame relay switch {name} is always-on
  Running on server {host} with port {port}
  Local ID is {id} and node ID is {node_id}
  Hardware is Dynamips emulated simple Frame relay switch
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           host=self._compute.name(),
           port=self._compute.port())

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "   Port {} is empty\n".format(port.name())
            else:
                port_info += "   Port {name} {description}\n".format(name=port.name(),
                                                                     description=port.description())

            for source, destination in self._settings["mappings"].items():
                source_port, source_dlci = source.split(":")
                destination_port, destination_dlci = destination.split(":")

                if port.name() == source_port or port.name() == destination_port:
                    if port.name() == source_port:
                        dlci1 = source_dlci
                        port = destination_port
                        dlci2 = destination_dlci
                    else:
                        dlci1 = destination_dlci
                        port = source_port
                        dlci2 = source_dlci
                    port_info += "      incoming DLCI {dlci1} is switched to port {port} outgoing DLCI {dlci2}\n".format(dlci1=dlci1,
                                                                                                                         port=port,
                                                                                                                         dlci2=dlci2)
                    break

        return info + port_info

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.frame_relay_switch_configuration_page import FrameRelaySwitchConfigurationPage
        return FrameRelaySwitchConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/frame_relay_switch.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.switches]

    def __str__(self):

        return "Frame Relay switch"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/docker/ui/docker_vm_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockerVMPreferencesPageWidget(object):
    def setupUi(self, DockerVMPreferencesPageWidget):
        DockerVMPreferencesPageWidget.setObjectName("DockerVMPreferencesPageWidget")
        DockerVMPreferencesPageWidget.resize(575, 435)
        DockerVMPreferencesPageWidget.setAccessibleDescription("")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DockerVMPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(DockerVMPreferencesPageWidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.uiDockerVMsTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDockerVMsTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiDockerVMsTreeWidget.setSizePolicy(sizePolicy)
        self.uiDockerVMsTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiDockerVMsTreeWidget.setFont(font)
        self.uiDockerVMsTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiDockerVMsTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiDockerVMsTreeWidget.setRootIsDecorated(False)
        self.uiDockerVMsTreeWidget.setObjectName("uiDockerVMsTreeWidget")
        self.uiDockerVMsTreeWidget.headerItem().setText(0, "1")
        self.uiDockerVMsTreeWidget.header().setVisible(False)
        self.widget = QtWidgets.QWidget(self.splitter)
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiDockerVMInfoTreeWidget = QtWidgets.QTreeWidget(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDockerVMInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiDockerVMInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiDockerVMInfoTreeWidget.setIndentation(10)
        self.uiDockerVMInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiDockerVMInfoTreeWidget.setObjectName("uiDockerVMInfoTreeWidget")
        self.uiDockerVMInfoTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiDockerVMInfoTreeWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiNewDockerVMPushButton = QtWidgets.QPushButton(self.widget)
        self.uiNewDockerVMPushButton.setObjectName("uiNewDockerVMPushButton")
        self.horizontalLayout.addWidget(self.uiNewDockerVMPushButton)
        self.uiCopyDockerVMPushButton = QtWidgets.QPushButton(self.widget)
        self.uiCopyDockerVMPushButton.setEnabled(False)
        self.uiCopyDockerVMPushButton.setObjectName("uiCopyDockerVMPushButton")
        self.horizontalLayout.addWidget(self.uiCopyDockerVMPushButton)
        self.uiEditDockerVMPushButton = QtWidgets.QPushButton(self.widget)
        self.uiEditDockerVMPushButton.setEnabled(False)
        self.uiEditDockerVMPushButton.setObjectName("uiEditDockerVMPushButton")
        self.horizontalLayout.addWidget(self.uiEditDockerVMPushButton)
        self.uiDeleteDockerVMPushButton = QtWidgets.QPushButton(self.widget)
        self.uiDeleteDockerVMPushButton.setEnabled(False)
        self.uiDeleteDockerVMPushButton.setObjectName("uiDeleteDockerVMPushButton")
        self.horizontalLayout.addWidget(self.uiDeleteDockerVMPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(DockerVMPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(DockerVMPreferencesPageWidget)
        DockerVMPreferencesPageWidget.setTabOrder(self.uiNewDockerVMPushButton, self.uiDeleteDockerVMPushButton)

    def retranslateUi(self, DockerVMPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        DockerVMPreferencesPageWidget.setWindowTitle(_translate("DockerVMPreferencesPageWidget", "Docker containers"))
        DockerVMPreferencesPageWidget.setAccessibleName(_translate("DockerVMPreferencesPageWidget", "Docker container templates"))
        self.uiDockerVMInfoTreeWidget.headerItem().setText(0, _translate("DockerVMPreferencesPageWidget", "1"))
        self.uiDockerVMInfoTreeWidget.headerItem().setText(1, _translate("DockerVMPreferencesPageWidget", "2"))
        self.uiNewDockerVMPushButton.setText(_translate("DockerVMPreferencesPageWidget", "&New"))
        self.uiCopyDockerVMPushButton.setText(_translate("DockerVMPreferencesPageWidget", "&Copy"))
        self.uiEditDockerVMPushButton.setText(_translate("DockerVMPreferencesPageWidget", "&Edit"))
        self.uiDeleteDockerVMPushButton.setText(_translate("DockerVMPreferencesPageWidget", "&Delete"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

from gns3.qt import QtWidgets
from gns3.compute_manager import ComputeManager
from gns3.controller import Controller


class VMWizard(QtWidgets.QWizard):
    """
    Base class for VM wizard.

    :param devices: List of existing device for this type
    :param parent: parent widget
    """

    def __init__(self, devices, parent):
        super().__init__(parent)
        self.setupUi(self)

        self.setModal(True)

        self._devices = devices
        self._allow_dynamic_compute_allocation = True
        self._local_server_disable = False

        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        self.uiRemoteRadioButton.toggled.connect(self._remoteServerToggledSlot)
        if hasattr(self, "uiVMRadioButton"):
            self.uiVMRadioButton.toggled.connect(self._vmToggledSlot)

        self.uiLocalRadioButton.toggled.connect(self._localToggledSlot)
        if Controller.instance().isRemote():
            self.uiLocalRadioButton.setText("Run device on the main server")

        # By default we use the local server
        self._compute_id = "local"
        self.uiLocalRadioButton.setChecked(True)
        self._localToggledSlot(True)

        if len(ComputeManager.instance().computes()) == 1:
            # skip the server page if we use the first server
            self.setStartId(1)

    def _vmToggledSlot(self, checked):
        """
        Slot for when the VM radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    def _remoteServerToggledSlot(self, checked):
        """
        Slot for when the remote server radio button is toggled.

        :param checked: either the button is checked or not
        """

        if checked:
            self.uiRemoteServersGroupBox.setEnabled(True)
            self.uiRemoteServersComboBox.setEnabled(True)
            self.uiRemoteServersGroupBox.show()

    def _localToggledSlot(self, checked):
        """
        Slot for when the local server radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    def initializePage(self, page_id):

        if self.page(page_id) == self.uiServerWizardPage:
            self.uiRemoteServersComboBox.clear()

            self.uiRemoteRadioButton.setEnabled(False)
            if hasattr(self, "uiVMRadioButton"):
                self.uiVMRadioButton.setEnabled(False)
            self.uiLocalRadioButton.setEnabled(False)
            if self._allow_dynamic_compute_allocation:
                self.uiRemoteServersComboBox.addItem("Any server", None)
            for compute in ComputeManager.instance().computes():
                if compute.id() == "local":
                    self.uiLocalRadioButton.setEnabled(True)
                elif compute.id() == "vm":
                    if hasattr(self, "uiVMRadioButton"):
                        self.uiVMRadioButton.setEnabled(True)
                else:
                    self.uiRemoteRadioButton.setEnabled(True)
                    self.uiRemoteServersComboBox.addItem(compute.name(), compute.id())

            if self.uiLocalRadioButton.isEnabled() and not self._local_server_disable:
                self.uiLocalRadioButton.setChecked(True)
            elif hasattr(self, "uiVMRadioButton") and self.uiVMRadioButton.isEnabled():
                self.uiVMRadioButton.setChecked(True)
            else:
                if self.uiRemoteRadioButton.isEnabled():
                    self.uiRemoteRadioButton.setChecked(True)
                else:
                    self.uiLocalRadioButton.setChecked(True)

    def _disableLocalServer(self):
        """
        Turn off the local server
        """
        self._local_server_disable = True
        self.uiLocalRadioButton.hide()
        self.uiLocalRadioButton.setEnabled(False)
        self.setStartId(0)

    def validateCurrentPage(self):
        if hasattr(self, "uiNameWizardPage") and self.currentPage() == self.uiNameWizardPage:
            name = self.uiNameLineEdit.text()
            for device in self._devices.values():
                if device["name"] == name:
                    QtWidgets.QMessageBox.critical(self, "Name", "{} is already used, please choose another name".format(name))
                    return False
        elif self.currentPage() == self.uiServerWizardPage:
            # If the local button is not visible it's because it's not supported
            if self.uiLocalRadioButton.isChecked() and self.uiLocalRadioButton.isHidden():
                QtWidgets.QMessageBox.critical(self, "New device", "Please configure before the GNS3 VM in order to use this device.")
                return False

            if self.uiRemoteRadioButton.isChecked():
                if self.uiRemoteServersComboBox.count() == 0:
                    QtWidgets.QMessageBox.critical(self, "Remote server", "There is no remote server registered in your preferences")
                    return False
                self._compute_id = self.uiRemoteServersComboBox.itemData(self.uiRemoteServersComboBox.currentIndex())
            elif hasattr(self, "uiVMRadioButton") and self.uiVMRadioButton.isChecked():
                self._compute_id = "vm"
            else:
                if self.uiLocalRadioButton.isEnabled():
                    self._compute_id = "local"
                else:
                    QtWidgets.QMessageBox.critical(self, "Server", "No available server support this type of node. You probably need to setup the GNS3 VM")
                    return False
        return True
